{"namespace": "mllint.avro",
    "type": "record",
    "name": "Observation",
    "fields": [
        {"name": "category", "type": "string"},
        {"name": "source_file", "type": "string"},
        {"name": "source_code", "type": "string"},
        {"name": "finder", "type": "string"},
        {"name": "finder_rule", "type": "string"}
        ]
}